@@ -231,11 +231,23 @@ class Layer:
             dill.dump(self, f)
 
 
-class WordVectors:
+class WordVectors(Layer):
+    parallel = False
+    trainable = False
+    document_wise = False
+
     def __init__(
         self,
-        model=None
+        model=None,
+        word2id=None,
+        id2word=None,
+        weights=None,
+        name=None,
+        verbose=True,
+        parallel=False
     ):
+        super().__init__(None, None, name, verbose, parallel)
+
         if model is not None:
             from tqdm import tqdm
             self.id2word = {}
@@ -249,6 +261,36 @@ class WordVectors:
                 self.word2id[word] = i
                 self.id2word[i] = word
             self.weights = np.vstack(weights)
+        else:
+            assert weights is not None
+            if word2id is not None:
+                if id2word is None:
+                    id2word = {i: word for word, i in word2id.items()}
+            elif id2word is not None:
+                word2id = {word: i for i, word in id2word.items()}
+            self.word2id = word2id
+            self.id2word = id2word
+            self.weights = weights
+
+    def fit_to_sequence(self, seq):
+        id2word = {i: word for word, i in seq.word2id.items()}
+        dim = self.weights.shape[1]
+        dtype = self.weights.dtype
+
+        new_weights = [np.zeros(dim, dtype=dtype)]
+        for i in range(1, max(id2word.keys()) + 1):
+            word = id2word[i]
+            # vec = np.zeros(dim, dtype=dtype)
+
+            weight_id = self.word2id.get(word)
+            if weight_id is None:
+                vec = np.zeros(dim, dtype=dtype)
+            else:
+                vec = self.weights[weight_id]
+            new_weights.append(vec)
+        new_weights = np.array(new_weights)
+
+        return WordVectors(id2word=id2word, weights=new_weights)
 
     def unload(self):
         pass
@@ -264,6 +306,20 @@ class WordVectors:
                 np.linalg.norm(self.weights, axis=1)[:, None].clip(1e-6, None)
             )
 
+    def process_series(self, series):
+        dim = self.weights.shape[1]
+        dtype = self.weights.dtype
+        nan = np.empty(dim, dtype=dtype)
+        nan[:] = np.nan
+        X = []
+        for x in series:
+            idx = self.word2id.get(x)
+            if idx is not None:
+                X.append(self.weights[idx])
+            else:
+                X.append(nan)
+        return np.array(X)
+
     def save(self, filename):
         import dill
         with open(filename, "wb") as f:
@@ -274,7 +330,7 @@ class WordVectors:
             yield key
 
     def __len__(self):
-        return self.num_words
+        return len(self.word2id)
 
     def __contains__(self, key):
         return key in self.word2id
@@ -312,3 +368,20 @@ def to_matrix(series):
         elif issparse(series[0]):
             series = vstack(series.tolist())
     return series
+
+
+def fit(model, nlp, series, y, batch_size=100, epochs=5):
+    import math
+    n = len(series)
+    n_batches = math.ceil(n / batch_size)
+
+    def get_data():
+        for _ in range(epochs):
+            for i in range(n_batches):
+                start = i * batch_size
+                end = (i + 1) * batch_size
+                X_batch = nlp(series.iloc[start:end])
+                y_batch = y[start:end]
+                yield X_batch, y_batch
+
+    model.fit(get_data(), epochs=5, steps_per_epoch=n_batches)
