@@ -1,24 +1,20 @@
 from .classifier import (MLP, RNN, SVM, TFMLP, AdaBoost, Keras, RandomForest,
                          Transformer, XGBoost)
 from .embedding import CountVectorizer, Doc2Vec, OneHot, Sequence, TfIdf
-from .linguistics import (Contract, FindAll, Lemmatize, NGram, Phrase,
-                          Snowball, Sub, Tokenize)
+from .linguistics import (NER, Contract, FindAll, LangDetect, Lemmatize, NGram,
+                          Phrase, Snowball, Sub, Tokenize)
 from .multi import Merge
 from .preprocessing import Normalize
 from .reduction import NMF, PCA, SVD, UMAP, RandomizedSVD
 from .regressor import KerasRegressor, MLPRegressor
-from .special import DomainName, Lambda, SplitHashtag
-
-# retro compatibility
-Lemmatizer = Lemmatize
-
+from .special import Argmax, DomainName, Lambda, SplitHashtag
 
 __all__ = [
     "Tokenize", "Snowball", "Phrase", "Sub", "Contract", "FindAll",
-    "TfIdf", "CountVectorizer", "Sequence", "OneHot", "Doc2Vec",
-    "SVD", "PCA", "NMF", "UMAP", "Lemmatize", "NGram",
+    "LangDetect", "TfIdf", "CountVectorizer", "Sequence", "OneHot",
+    "Doc2Vec", "SVD", "PCA", "NMF", "UMAP", "Lemmatize", "NGram",
     "RandomForest", "AdaBoost", "SVM", "MLP", "Keras", "Transformer",
     "XGBoost", "Merge", "Normalize", "SplitHashtag", "TFMLP",
     "MLPRegressor", "KerasRegressor", "Lambda", "RNN", "DomainName",
-    "RandomizedSVD"
+    "RandomizedSVD", "NER", "Argmax"
 ]
